@@ -1,20 +1,16 @@
 use crate::options::Config;
-use crate::parse::{parse_basic_auth, parse_duration_secs, parse_headers, parse_remaps};
+use crate::parse::{parse_duration_secs, parse_headers, parse_remaps};
 use anyhow::{Context, Result};
-use headers::HeaderMapExt;
 use http::StatusCode;
 use lychee_lib::{Client, ClientBuilder};
 use regex::RegexSet;
+use reqwest_cookie_store::CookieStoreMutex;
+use std::sync::Arc;
 use std::{collections::HashSet, str::FromStr};
 
 /// Creates a client according to the command-line config
-pub(crate) fn create(cfg: &Config) -> Result<Client> {
-    let mut headers = parse_headers(&cfg.header)?;
-    if let Some(auth) = &cfg.basic_auth {
-        let auth_header = parse_basic_auth(auth)?;
-        headers.typed_insert(auth_header);
-    }
-
+pub(crate) fn create(cfg: &Config, cookie_jar: Option<&Arc<CookieStoreMutex>>) -> Result<Client> {
+    let headers = parse_headers(&cfg.header)?;
     let timeout = parse_duration_secs(cfg.timeout);
     let retry_wait_time = parse_duration_secs(cfg.retry_wait_time);
     let method: reqwest::Method = reqwest::Method::from_str(&cfg.method.to_uppercase())?;
@@ -30,15 +26,31 @@ pub(crate) fn create(cfg: &Config) -> Result<Client> {
         cfg.scheme.clone()
     };
 
-    let accepted = match cfg.accept {
-        Some(ref accepted) => {
-            let accepted: Result<HashSet<_>, _> = accepted
-                .iter()
-                .map(|code| StatusCode::from_u16(*code))
-                .collect();
-            Some(accepted?)
-        }
-        None => None,
+    let accepted = cfg
+        .accept
+        .clone()
+        .into_set()
+        .iter()
+        .map(|value| StatusCode::from_u16(*value))
+        .collect::<Result<HashSet<_>, _>>()?;
+
+    // `exclude_mail` will be removed in 1.0. Until then, we need to support it.
+    // Therefore, we need to check if both `include_mail` and `exclude_mail` are set to `true`
+    // and return an error if that's the case.
+    if cfg.include_mail && cfg.exclude_mail {
+        return Err(anyhow::anyhow!(
+            "Cannot set both `include-mail` and `exclude-mail` to true"
+        ));
+    }
+
+    // By default, clap sets `exclude_mail` to `false`.
+    // Therefore, we need to check if `exclude_mail` is explicitly set to
+    // `true`. If so, we need to set `include_mail` to `false`.
+    // Otherwise, we use the value of `include_mail`.
+    let include_mail = if cfg.exclude_mail {
+        false
+    } else {
+        cfg.include_mail
     };
 
     ClientBuilder::builder()
@@ -49,7 +61,7 @@ pub(crate) fn create(cfg: &Config) -> Result<Client> {
         .exclude_private_ips(cfg.exclude_private)
         .exclude_link_local_ips(cfg.exclude_link_local)
         .exclude_loopback_ips(cfg.exclude_loopback)
-        .exclude_mail(cfg.exclude_mail)
+        .include_mail(include_mail)
         .max_redirects(cfg.max_redirects)
         .user_agent(cfg.user_agent.clone())
         .allow_insecure(cfg.insecure)
@@ -61,6 +73,8 @@ pub(crate) fn create(cfg: &Config) -> Result<Client> {
         .schemes(HashSet::from_iter(schemes))
         .accepted(accepted)
         .require_https(cfg.require_https)
+        .cookie_jar(cookie_jar.cloned())
+        .include_fragments(cfg.include_fragments)
         .build()
         .client()
         .context("Failed to create request client")
