@@ -1,10 +1,10 @@
 use std::{
-    collections::HashMap,
     fmt::{self, Display},
+    time::Duration,
 };
 
 use crate::{
-    color::{color, BOLD_GREEN, BOLD_PINK, BOLD_YELLOW, NORMAL, PINK},
+    color::{color, BOLD_GREEN, BOLD_PINK, BOLD_YELLOW, DIM, NORMAL},
     formatters::color_response,
     stats::ResponseStats,
 };
@@ -15,27 +15,6 @@ use anyhow::Result;
 
 struct CompactResponseStats(ResponseStats);
 
-// Helper function, which prints the detailed list of errors
-pub(crate) fn print_errors(stats: &ResponseStats) -> String {
-    let mut errors = HashMap::new();
-    errors.insert(
-        "HTTP",
-        stats.errors - stats.redirects - stats.timeouts - stats.unknown,
-    );
-    errors.insert("Redirects", stats.redirects);
-    errors.insert("Timeouts", stats.timeouts);
-    errors.insert("Unknown", stats.unknown);
-
-    // Creates an output like `(HTTP:3|Timeouts:1|Unknown:1)`
-    let mut err: Vec<_> = errors
-        .into_iter()
-        .filter(|(_, v)| *v > 0)
-        .map(|(k, v)| format!("{k}:{v}"))
-        .collect();
-    err.sort();
-    err.join("|")
-}
-
 impl Display for CompactResponseStats {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let stats = &self.0;
@@ -59,20 +38,29 @@ impl Display for CompactResponseStats {
             for response in responses {
                 writeln!(f, "{}", color_response(response))?;
             }
+
+            if let Some(suggestions) = &stats.suggestion_map.get(source) {
+                writeln!(f, "\n\u{2139} Suggestions")?;
+                for suggestion in *suggestions {
+                    writeln!(f, "{suggestion}")?;
+                }
+            }
+
             writeln!(f)?;
         }
 
         color!(f, NORMAL, "\u{1F50D} {} Total", stats.total)?;
+
+        // show duration (in a human readable format), e.g. 2m 30s
+        let duration = Duration::from_secs(stats.duration_secs);
+        color!(f, DIM, " (in {})", humantime::format_duration(duration))?;
+
         color!(f, BOLD_GREEN, " \u{2705} {} OK", stats.successful)?;
 
         let total_errors = stats.errors;
 
         let err_str = if total_errors == 1 { "Error" } else { "Errors" };
         color!(f, BOLD_PINK, " \u{1f6ab} {} {}", total_errors, err_str)?;
-        if total_errors > 0 {
-            write!(f, " ")?;
-            color!(f, PINK, "({})", print_errors(stats))?;
-        }
         if stats.excludes > 0 {
             color!(f, BOLD_YELLOW, " \u{1F4A4} {} Excluded", stats.excludes)?;
         }
