@@ -1,11 +1,11 @@
 use std::collections::{HashMap, HashSet};
 
+use crate::archive::Suggestion;
 use lychee_lib::{CacheStatus, InputSource, Response, ResponseBody, Status};
 use serde::Serialize;
 
 #[derive(Default, Serialize, Debug)]
 pub(crate) struct ResponseStats {
-    pub(crate) detailed_stats: bool,
     pub(crate) total: usize,
     pub(crate) successful: usize,
     pub(crate) unknown: usize,
@@ -17,7 +17,10 @@ pub(crate) struct ResponseStats {
     pub(crate) cached: usize,
     pub(crate) success_map: HashMap<InputSource, HashSet<ResponseBody>>,
     pub(crate) fail_map: HashMap<InputSource, HashSet<ResponseBody>>,
+    pub(crate) suggestion_map: HashMap<InputSource, HashSet<Suggestion>>,
     pub(crate) excluded_map: HashMap<InputSource, HashSet<ResponseBody>>,
+    pub(crate) duration_secs: u64,
+    pub(crate) detailed_stats: bool,
 }
 
 impl ResponseStats {
@@ -75,7 +78,7 @@ impl ResponseStats {
 
     #[inline]
     pub(crate) const fn is_success(&self) -> bool {
-        self.total == self.successful + self.excludes
+        self.total == self.successful + self.excludes + self.unsupported
     }
 
     #[inline]
