@@ -1,6 +1,7 @@
 #[cfg(test)]
 mod cli {
     use std::{
+        collections::{HashMap, HashSet},
         error::Error,
         fs::{self, File},
         io::Write,
@@ -8,9 +9,20 @@ mod cli {
     };
 
     use assert_cmd::Command;
+    use assert_json_diff::assert_json_include;
     use http::StatusCode;
-    use predicates::str::{contains, is_empty};
+    use lychee_lib::{InputSource, ResponseBody};
+    use predicates::{
+        prelude::predicate,
+        str::{contains, is_empty},
+    };
+    use pretty_assertions::assert_eq;
+    use regex::Regex;
+    use serde::Serialize;
+    use serde_json::Value;
+    use tempfile::NamedTempFile;
     use uuid::Uuid;
+    use wiremock::{matchers::basic_auth, Mock, ResponseTemplate};
 
     type Result<T> = std::result::Result<T, Box<dyn Error>>;
 
@@ -23,8 +35,8 @@ mod cli {
     macro_rules! mock_server {
         ($status:expr $(, $func:tt ($($arg:expr),*))*) => {{
             let mock_server = wiremock::MockServer::start().await;
-            let template = wiremock::ResponseTemplate::new(http::StatusCode::from($status));
-            let template = template$(.$func($($arg),*))*;
+            let response_template = wiremock::ResponseTemplate::new(http::StatusCode::from($status));
+            let template = response_template$(.$func($($arg),*))*;
             wiremock::Mock::given(wiremock::matchers::method("GET")).respond_with(template).mount(&mock_server).await;
             mock_server
         }};
@@ -48,53 +60,35 @@ mod cli {
         Command::cargo_bin(env!("CARGO_PKG_NAME")).expect("Couldn't get cargo package name")
     }
 
-    /// Helper function to get the path to the fixtures directory.
-    fn fixtures_path() -> PathBuf {
+    /// Helper function to get the root path of the project.
+    fn root_path() -> PathBuf {
         Path::new(env!("CARGO_MANIFEST_DIR"))
             .parent()
             .unwrap()
-            .join("fixtures")
+            .to_path_buf()
     }
 
-    #[derive(Default)]
+    /// Helper function to get the path to the fixtures directory.
+    fn fixtures_path() -> PathBuf {
+        root_path().join("fixtures")
+    }
+
+    #[derive(Default, Serialize)]
     struct MockResponseStats {
+        detailed_stats: bool,
         total: usize,
         successful: usize,
-        failures: usize,
         unknown: usize,
+        unsupported: usize,
         timeouts: usize,
         redirects: usize,
         excludes: usize,
         errors: usize,
         cached: usize,
-    }
-
-    impl MockResponseStats {
-        fn to_json_str(&self) -> String {
-            format!(
-                r#"{{
-  "total": {},
-  "successful": {},
-  "failures": {},
-  "unknown": {},
-  "timeouts": {},
-  "redirects": {},
-  "excludes": {},
-  "errors": {},
-  "cached": {},
-  "fail_map": {{}}
-}}"#,
-                self.total,
-                self.successful,
-                self.failures,
-                self.unknown,
-                self.timeouts,
-                self.redirects,
-                self.excludes,
-                self.errors,
-                self.cached
-            )
-        }
+        success_map: HashMap<InputSource, HashSet<ResponseBody>>,
+        fail_map: HashMap<InputSource, HashSet<ResponseBody>>,
+        suggestion_map: HashMap<InputSource, HashSet<ResponseBody>>,
+        excluded_map: HashMap<InputSource, HashSet<ResponseBody>>,
     }
 
     /// Helper macro to test the output of the JSON format.
@@ -104,17 +98,71 @@ mod cli {
             let test_path = fixtures_path().join($test_file);
             let outfile = format!("{}.json", uuid::Uuid::new_v4());
 
-            let expected = $expected.to_json_str();
-
             cmd$(.arg($arg))*.arg("--output").arg(&outfile).arg("--format").arg("json").arg(test_path).assert().success();
 
             let output = std::fs::read_to_string(&outfile)?;
-            assert_eq!(output, expected);
             std::fs::remove_file(outfile)?;
+
+            let actual: Value = serde_json::from_str(&output)?;
+            let expected: Value = serde_json::to_value(&$expected)?;
+
+            assert_json_include!(actual: actual, expected: expected);
             Ok(())
         }};
     }
 
+    /// JSON-formatted output should always be valid JSON.
+    /// Additional hints and error messages should be printed to `stderr`.
+    /// See https://github.com/lycheeverse/lychee/issues/1355
+    #[test]
+    fn test_valid_json_output_to_stdout_on_error() -> Result<()> {
+        let test_path = fixtures_path().join("TEST_GITHUB_404.md");
+
+        let mut cmd = main_command();
+        cmd.arg("--format")
+            .arg("json")
+            .arg(test_path)
+            .assert()
+            .failure()
+            .code(2);
+
+        let output = cmd.output()?;
+
+        // Check that the output is valid JSON
+        assert!(serde_json::from_slice::<Value>(&output.stdout).is_ok());
+        Ok(())
+    }
+
+    #[test]
+    fn test_detailed_json_output_on_error() -> Result<()> {
+        let test_path = fixtures_path().join("TEST_DETAILED_JSON_OUTPUT_ERROR.md");
+
+        let mut cmd = main_command();
+        cmd.arg("--format")
+            .arg("json")
+            .arg(&test_path)
+            .assert()
+            .failure()
+            .code(2);
+
+        let output = cmd.output()?;
+
+        // Check that the output is valid JSON
+        assert!(serde_json::from_slice::<Value>(&output.stdout).is_ok());
+
+        // Parse site error status from the fail_map
+        let output_json = serde_json::from_slice::<Value>(&output.stdout).unwrap();
+        let site_error_status = &output_json["fail_map"][&test_path.to_str().unwrap()][0]["status"];
+
+        assert_eq!(
+            "error:0A000086:SSL routines:tls_post_process_server_certificate:\
+            certificate verify failed:../ssl/statem/statem_clnt.c:1883: \
+            (certificate has expired)",
+            site_error_status["details"]
+        );
+        Ok(())
+    }
+
     #[test]
     fn test_exclude_all_private() -> Result<()> {
         test_json_output!(
@@ -124,22 +172,34 @@ mod cli {
                 excludes: 7,
                 ..MockResponseStats::default()
             },
-            "--exclude-all-private",
-            "--verbose"
+            "--exclude-all-private"
         )
     }
 
     #[test]
-    fn test_exclude_email() -> Result<()> {
+    fn test_email() -> Result<()> {
         test_json_output!(
             "TEST_EMAIL.md",
             MockResponseStats {
-                total: 6,
-                excludes: 4,
-                successful: 2,
+                total: 5,
+                excludes: 0,
+                successful: 5,
                 ..MockResponseStats::default()
             },
-            "--exclude-mail"
+            "--include-mail"
+        )
+    }
+
+    #[test]
+    fn test_exclude_email_by_default() -> Result<()> {
+        test_json_output!(
+            "TEST_EMAIL.md",
+            MockResponseStats {
+                total: 5,
+                excludes: 3,
+                successful: 2,
+                ..MockResponseStats::default()
+            }
         )
     }
 
@@ -150,6 +210,7 @@ mod cli {
 
         cmd.arg("--dump")
             .arg(input)
+            .arg("--include-mail")
             .assert()
             .success()
             .stdout(contains("hello@example.org?subject=%5BHello%5D"));
@@ -164,6 +225,7 @@ mod cli {
 
         cmd.arg("--dump")
             .arg(input)
+            .arg("--include-mail")
             .assert()
             .success()
             .stdout(contains("hello@example.org?subject=%5BHello%5D"));
@@ -186,7 +248,7 @@ mod cli {
 
     /// Test unsupported URI schemes
     #[test]
-    fn test_unsupported_uri_schemes() {
+    fn test_unsupported_uri_schemes_are_ignored() {
         let mut cmd = main_command();
         let test_schemes_path = fixtures_path().join("TEST_SCHEMES.txt");
 
@@ -199,7 +261,7 @@ mod cli {
             .env_clear()
             .assert()
             .success()
-            .stdout(contains("2 Total"))
+            .stdout(contains("3 Total"))
             .stdout(contains("1 OK"))
             .stdout(contains("1 Excluded"));
     }
@@ -216,21 +278,56 @@ mod cli {
             .env_clear()
             .assert()
             .success()
-            .stdout(contains("3 Total"))
-            .stdout(contains("3 OK"));
+            .stdout(contains("4 Total"))
+            .stdout(contains("4 OK"));
     }
 
     #[test]
-    fn test_quirks() -> Result<()> {
-        test_json_output!(
-            "TEST_QUIRKS.txt",
-            MockResponseStats {
-                total: 3,
-                successful: 3,
-                excludes: 0,
-                ..MockResponseStats::default()
-            }
-        )
+    fn test_youtube_quirk() {
+        let url = "https://www.youtube.com/watch?v=NlKuICiT470&list=PLbWDhxwM_45mPVToqaIZNbZeIzFchsKKQ&index=7";
+
+        main_command()
+            .write_stdin(url)
+            .arg("--verbose")
+            .arg("--no-progress")
+            .arg("-")
+            .assert()
+            .success()
+            .stdout(contains("1 Total"))
+            .stdout(contains("1 OK"));
+    }
+
+    #[test]
+    fn test_crates_io_quirk() {
+        let url = "https://crates.io/crates/lychee";
+
+        main_command()
+            .write_stdin(url)
+            .arg("--verbose")
+            .arg("--no-progress")
+            .arg("-")
+            .assert()
+            .success()
+            .stdout(contains("1 Total"))
+            .stdout(contains("1 OK"));
+    }
+
+    #[test]
+    // Exclude Twitter links because they require login to view tweets.
+    // https://techcrunch.com/2023/06/30/twitter-now-requires-an-account-to-view-tweets/
+    // https://github.com/zedeus/nitter/issues/919
+    fn test_ignored_hosts() {
+        let url = "https://twitter.com/zarfeblong/status/1339742840142872577";
+
+        main_command()
+            .write_stdin(url)
+            .arg("--verbose")
+            .arg("--no-progress")
+            .arg("-")
+            .assert()
+            .success()
+            .stdout(contains("1 Total"))
+            .stdout(contains("1 Excluded"));
     }
 
     #[tokio::test]
@@ -298,7 +395,6 @@ mod cli {
                 excludes: 0,
                 ..MockResponseStats::default()
             },
-            "--verbose",
             // Two requests to the same URI may be executed in parallel. As a
             // result, the response might not be cached and the test would be
             // flaky. Therefore limit the concurrency to one request at a time.
@@ -319,9 +415,9 @@ mod cli {
             .failure()
             .code(2)
             .stdout(contains(
-                "https://github.com/mre/idiomatic-rust-doesnt-exist-man | Network error: Not Found",
+                "✗ [404] https://github.com/mre/idiomatic-rust-doesnt-exist-man | Failed: Network error: Not Found"
             ))
-            .stdout(contains(
+            .stderr(contains(
                 "There were issues with Github URLs. You could try setting a Github token and running lychee again.",
             ));
     }
@@ -446,23 +542,15 @@ mod cli {
     /// Test formatted file output
     #[test]
     fn test_formatted_file_output() -> Result<()> {
-        let mut cmd = main_command();
-        let test_path = fixtures_path().join("TEST.md");
-        let outfile = format!("{}.json", Uuid::new_v4());
-
-        cmd.arg("--output")
-            .arg(&outfile)
-            .arg("--format")
-            .arg("json")
-            .arg(test_path)
-            .assert()
-            .success();
-
-        let expected = r#"{"total":11,"successful":11,"failures":0,"unknown":0,"timeouts":0,"redirects":0,"excludes":0,"errors":0,"cached":0,"fail_map":{}}"#;
-        let output = fs::read_to_string(&outfile)?;
-        assert_eq!(output.split_whitespace().collect::<String>(), expected);
-        fs::remove_file(outfile)?;
-        Ok(())
+        test_json_output!(
+            "TEST.md",
+            MockResponseStats {
+                total: 12,
+                successful: 10,
+                excludes: 2,
+                ..MockResponseStats::default()
+            }
+        )
     }
 
     /// Test writing output of `--dump` command to file
@@ -475,6 +563,7 @@ mod cli {
         cmd.arg("--output")
             .arg(&outfile)
             .arg("--dump")
+            .arg("--include-mail")
             .arg(test_path)
             .assert()
             .success();
@@ -485,7 +574,7 @@ mod cli {
         // Running the command from the command line will print 9 links,
         // because the actual `--dump` command filters out the two
         // http(s)://example.com links
-        assert_eq!(output.lines().count(), 11);
+        assert_eq!(output.lines().count(), 12);
         fs::remove_file(outfile)?;
         Ok(())
     }
@@ -501,7 +590,7 @@ mod cli {
             .arg(".*")
             .assert()
             .success()
-            .stdout(contains("11 Excluded"));
+            .stdout(contains("12 Excluded"));
 
         Ok(())
     }
@@ -514,61 +603,11 @@ mod cli {
         cmd.arg(test_path)
             .arg("--exclude")
             .arg("https://en.wikipedia.org/*")
+            .arg("--exclude")
             .arg("https://ldra.com/")
             .assert()
             .success()
-            .stdout(contains("2 Excluded"));
-
-        Ok(())
-    }
-
-    #[test]
-    fn test_exclude_file() -> Result<()> {
-        let mut cmd = main_command();
-        let test_path = fixtures_path().join("TEST.md");
-        let excludes_path = fixtures_path().join("TEST_EXCLUDE_1.txt");
-
-        cmd.arg(test_path)
-            .arg("--exclude-file")
-            .arg(excludes_path)
-            .assert()
-            .success()
-            .stdout(contains("2 Excluded"));
-
-        Ok(())
-    }
-
-    #[test]
-    fn test_multiple_exclude_files() -> Result<()> {
-        let mut cmd = main_command();
-        let test_path = fixtures_path().join("TEST.md");
-        let excludes_path1 = fixtures_path().join("TEST_EXCLUDE_1.txt");
-        let excludes_path2 = fixtures_path().join("TEST_EXCLUDE_2.txt");
-
-        cmd.arg(test_path)
-            .arg("--exclude-file")
-            .arg(excludes_path1)
-            .arg(excludes_path2)
-            .assert()
-            .success()
-            .stdout(contains("3 Excluded"));
-
-        Ok(())
-    }
-
-    #[tokio::test]
-    async fn test_example_config() -> Result<()> {
-        let mock_server = mock_server!(StatusCode::OK);
-        let mut cmd = main_command();
-        cmd.arg("--config")
-            .arg("lychee.example.toml")
-            .arg("-")
-            .write_stdin(mock_server.uri())
-            .env_clear()
-            .assert()
-            .success()
-            .stdout(contains("1 Total"))
-            .stdout(contains("1 OK"));
+            .stdout(contains("4 Excluded"));
 
         Ok(())
     }
@@ -591,6 +630,41 @@ mod cli {
         Ok(())
     }
 
+    #[tokio::test]
+    async fn test_include_mail_config() -> Result<()> {
+        let test_mail_address = "mailto:hello-test@testingabc.io";
+
+        let mut config = NamedTempFile::new()?;
+        writeln!(config, "include_mail = false")?;
+
+        let mut cmd = main_command();
+        cmd.arg("--config")
+            .arg(config.path().to_str().unwrap())
+            .arg("-")
+            .write_stdin(test_mail_address)
+            .env_clear()
+            .assert()
+            .success()
+            .stdout(contains("1 Total"))
+            .stdout(contains("1 Excluded"));
+
+        let mut config = NamedTempFile::new()?;
+        writeln!(config, "include_mail = true")?;
+
+        let mut cmd = main_command();
+        cmd.arg("--config")
+            .arg(config.path().to_str().unwrap())
+            .arg("-")
+            .write_stdin(test_mail_address)
+            .env_clear()
+            .assert()
+            .failure()
+            .stdout(contains("1 Total"))
+            .stdout(contains("1 Error"));
+
+        Ok(())
+    }
+
     #[tokio::test]
     async fn test_cache_config() -> Result<()> {
         let mock_server = mock_server!(StatusCode::OK);
@@ -618,9 +692,40 @@ mod cli {
             .arg("-")
             .env_clear()
             .assert()
+            .failure()
+            .stderr(predicate::str::contains("Cannot load configuration file"))
+            .stderr(predicate::str::contains("Failed to parse"))
+            .stderr(predicate::str::contains("TOML parse error"))
+            .stderr(predicate::str::contains("expected newline"));
+    }
+
+    #[tokio::test]
+    async fn test_missing_config_error() {
+        let mock_server = mock_server!(StatusCode::OK);
+        let mut cmd = main_command();
+        cmd.arg("--config")
+            .arg("config.does.not.exist.toml")
+            .arg("-")
+            .write_stdin(mock_server.uri())
+            .env_clear()
+            .assert()
             .failure();
     }
 
+    #[tokio::test]
+    async fn test_config_example() {
+        let mock_server = mock_server!(StatusCode::OK);
+        let config = root_path().join("lychee.example.toml");
+        let mut cmd = main_command();
+        cmd.arg("--config")
+            .arg(config)
+            .arg("-")
+            .write_stdin(mock_server.uri())
+            .env_clear()
+            .assert()
+            .success();
+    }
+
     #[tokio::test]
     async fn test_config_smoketest() {
         let mock_server = mock_server!(StatusCode::OK);
@@ -635,6 +740,20 @@ mod cli {
             .success();
     }
 
+    #[tokio::test]
+    async fn test_config_accept() {
+        let mock_server = mock_server!(StatusCode::OK);
+        let config = fixtures_path().join("configs").join("accept.toml");
+        let mut cmd = main_command();
+        cmd.arg("--config")
+            .arg(config)
+            .arg("-")
+            .write_stdin(mock_server.uri())
+            .env_clear()
+            .assert()
+            .success();
+    }
+
     #[test]
     fn test_lycheeignore_file() -> Result<()> {
         let mut cmd = main_command();
@@ -698,6 +817,7 @@ mod cli {
             .current_dir(&base_path)
             .arg(dir.path().join("c.md"))
             .arg("--verbose")
+            .arg("--no-progress")
             .arg("--cache")
             .arg("--exclude")
             .arg(mock_server_exclude.uri());
@@ -712,34 +832,25 @@ mod cli {
             .assert()
             .stderr(contains(format!("[200] {}/\n", mock_server_ok.uri())))
             .stderr(contains(format!(
-                "[404] {}/ | Network error: Not Found\n",
+                "[404] {}/ | Failed: Network error: Not Found\n",
                 mock_server_err.uri()
-            )))
-            .stderr(contains(format!(
-                "[EXCLUDED] {}/ | Excluded\n",
-                mock_server_exclude.uri()
             )));
 
         // check content of cache file
         let data = fs::read_to_string(&cache_file)?;
         assert!(data.contains(&format!("{}/,200", mock_server_ok.uri())));
         assert!(data.contains(&format!("{}/,404", mock_server_err.uri())));
-        assert!(data.contains(&format!("{}/,Excluded", mock_server_exclude.uri())));
 
         // run again to verify cache behavior
         test_cmd
             .assert()
             .stderr(contains(format!(
-                "[200] {}/ | OK (cached)\n",
+                "[200] {}/ | Cached: OK (cached)\n",
                 mock_server_ok.uri()
             )))
             .stderr(contains(format!(
-                "[404] {}/ | Error (cached)\n",
+                "[404] {}/ | Cached: Error (cached)\n",
                 mock_server_err.uri()
-            )))
-            .stderr(contains(format!(
-                "[EXCLUDED] {}/ | Excluded\n",
-                mock_server_exclude.uri()
             )));
 
         // clear the cache file
@@ -748,48 +859,9 @@ mod cli {
         Ok(())
     }
 
-    #[tokio::test]
-    async fn test_skip_cache_unsupported() -> Result<()> {
-        let base_path = fixtures_path().join("cache");
-        let cache_file = base_path.join(LYCHEE_CACHE_FILE);
-
-        // Unconditionally remove cache file if it exists
-        let _ = fs::remove_file(&cache_file);
-
-        let unsupported_url = "slack://user".to_string();
-        let excluded_url = "https://example.com/";
-
-        // run first without cache to generate the cache file
-        main_command()
-            .current_dir(&base_path)
-            .write_stdin(format!("{unsupported_url}\n{excluded_url}"))
-            .arg("--cache")
-            .arg("--verbose")
-            .arg("--exclude")
-            .arg(excluded_url)
-            .arg("--")
-            .arg("-")
-            .assert()
-            .stderr(contains(format!(
-                "[IGNORED] {unsupported_url} | Unsupported Error creating request client\n"
-            )))
-            .stderr(contains(format!("[EXCLUDED] {excluded_url} | Excluded\n")));
-
-        // The cache file should be empty, because the only checked URL is
-        // unsupported and we don't want to cache that. It might be supported in
-        // future versions.
-        let buf = fs::read(&cache_file).unwrap();
-        assert!(buf.is_empty());
-
-        // clear the cache file
-        fs::remove_file(&cache_file)?;
-
-        Ok(())
-    }
-
     #[tokio::test]
     async fn test_lycheecache_accept_custom_status_codes() -> Result<()> {
-        let base_path = fixtures_path().join("cache");
+        let base_path = fixtures_path().join("cache_accept_custom_status_codes");
         let cache_file = base_path.join(LYCHEE_CACHE_FILE);
 
         // Unconditionally remove cache file if it exists
@@ -825,11 +897,11 @@ mod cli {
             .failure()
             .code(2)
             .stdout(contains(format!(
-                "[418] {}/ | Failed: Network error: I'm a teapot\n",
+                "[418] {}/ | Failed: Network error: I\'m a teapot",
                 mock_server_teapot.uri()
             )))
             .stdout(contains(format!(
-                "[500] {}/ | Failed: Network error: Internal Server Error\n",
+                "[500] {}/ | Failed: Network error: Internal Server Error",
                 mock_server_server_error.uri()
             )));
 
@@ -847,12 +919,12 @@ mod cli {
             .arg("418,500")
             .assert()
             .success()
-            .stdout(contains(format!(
-                "[418] {}/ | Cached: OK (cached)\n",
+            .stderr(contains(format!(
+                "[418] {}/ | Cached: OK (cached)",
                 mock_server_teapot.uri()
             )))
-            .stdout(contains(format!(
-                "[500] {}/ | Cached: OK (cached)\n",
+            .stderr(contains(format!(
+                "[500] {}/ | Cached: OK (cached)",
                 mock_server_server_error.uri()
             )));
 
@@ -862,6 +934,100 @@ mod cli {
         Ok(())
     }
 
+    #[tokio::test]
+    async fn test_skip_cache_unsupported() -> Result<()> {
+        let base_path = fixtures_path().join("cache");
+        let cache_file = base_path.join(LYCHEE_CACHE_FILE);
+
+        // Unconditionally remove cache file if it exists
+        let _ = fs::remove_file(&cache_file);
+
+        let unsupported_url = "slack://user".to_string();
+        let excluded_url = "https://example.com/";
+
+        // run first without cache to generate the cache file
+        main_command()
+            .current_dir(&base_path)
+            .write_stdin(format!("{unsupported_url}\n{excluded_url}"))
+            .arg("--cache")
+            .arg("--verbose")
+            .arg("--no-progress")
+            .arg("--exclude")
+            .arg(excluded_url)
+            .arg("--")
+            .arg("-")
+            .assert()
+            .stderr(contains(format!(
+                "[IGNORED] {unsupported_url} | Unsupported: Error creating request client"
+            )))
+            .stderr(contains(format!("[EXCLUDED] {excluded_url} | Excluded\n")));
+
+        // The cache file should be empty, because the only checked URL is
+        // unsupported and we don't want to cache that. It might be supported in
+        // future versions.
+        let buf = fs::read(&cache_file).unwrap();
+        assert!(buf.is_empty());
+
+        // clear the cache file
+        fs::remove_file(&cache_file)?;
+
+        Ok(())
+    }
+
+    /// Unknown status codes should be skipped and not cached by default
+    /// The reason is that we don't know if they are valid or not
+    /// and even if they are invalid, we don't know if they will be valid in the
+    /// future.
+    ///
+    /// Since we cannot test this with our mock server (because hyper panics on
+    /// invalid status codes) we use LinkedIn as a test target.
+    ///
+    /// Unfortunately, LinkedIn does not always return 999, so this is a flaky
+    /// test. We only check that the cache file doesn't contain any invalid
+    /// status codes.
+    #[tokio::test]
+    async fn test_skip_cache_unknown_status_code() -> Result<()> {
+        let base_path = fixtures_path().join("cache");
+        let cache_file = base_path.join(LYCHEE_CACHE_FILE);
+
+        // Unconditionally remove cache file if it exists
+        let _ = fs::remove_file(&cache_file);
+
+        // https://linkedin.com returns 999 for unknown status codes
+        // use this as a test target
+        let unknown_url = "https://www.linkedin.com/company/corrode";
+
+        // run first without cache to generate the cache file
+        main_command()
+            .current_dir(&base_path)
+            .write_stdin(unknown_url.to_string())
+            .arg("--cache")
+            .arg("--verbose")
+            .arg("--no-progress")
+            .arg("--")
+            .arg("-")
+            .assert()
+            // LinkedIn does not always return 999, so we cannot check for that
+            // .stderr(contains(format!("[999] {unknown_url} | Unknown status")))
+            ;
+
+        // If the status code was 999, the cache file should be empty
+        // because we do not want to cache unknown status codes
+        let buf = fs::read(&cache_file).unwrap();
+        if !buf.is_empty() {
+            let data = String::from_utf8(buf)?;
+            // The cache file should not contain any invalid status codes
+            // In that case, we expect a single entry with status code 200
+            assert!(!data.contains("999"));
+            assert!(data.contains("200"));
+        }
+
+        // clear the cache file
+        fs::remove_file(&cache_file)?;
+
+        Ok(())
+    }
+
     #[test]
     fn test_verbatim_skipped_by_default() -> Result<()> {
         let mut cmd = main_command();
@@ -892,6 +1058,19 @@ mod cli {
 
         Ok(())
     }
+    #[tokio::test]
+    async fn test_verbatim_skipped_by_default_via_file() -> Result<()> {
+        let file = fixtures_path().join("TEST_VERBATIM.html");
+
+        main_command()
+            .arg("--dump")
+            .arg(file)
+            .assert()
+            .success()
+            .stdout(is_empty());
+
+        Ok(())
+    }
 
     #[tokio::test]
     async fn test_verbatim_skipped_by_default_via_remote_url() -> Result<()> {
@@ -926,7 +1105,6 @@ mod cli {
             .stdout(contains("http://www.example.com/samp"))
             .stdout(contains("http://www.example.com/kbd"))
             .stdout(contains("http://www.example.com/var"))
-            .stdout(contains("http://www.example.com/address"))
             .stdout(contains("http://www.example.com/script"));
         Ok(())
     }
@@ -1012,18 +1190,70 @@ mod cli {
             .arg("https://example.com http://127.0.0.1:8080")
             .arg("--remap")
             .arg("https://example.org https://staging.example.com")
+            .arg("--")
+            .arg("-")
+            .write_stdin("https://example.com\nhttps://example.org\nhttps://example.net\n")
+            .env_clear()
+            .assert()
+            .success()
+            .stdout(contains("http://127.0.0.1:8080/"))
+            .stdout(contains("https://staging.example.com/"))
+            .stdout(contains("https://example.net/"));
+
+        Ok(())
+    }
+
+    #[test]
+    #[ignore = "Skipping test until https://github.com/robinst/linkify/pull/58 is merged"]
+    fn test_remap_path() -> Result<()> {
+        let mut cmd = main_command();
+
+        cmd.arg("--dump")
             .arg("--remap")
             .arg("../../issues https://github.com/usnistgov/OSCAL/issues")
             .arg("--")
             .arg("-")
-            .write_stdin("file://../../issues\nhttps://example.com\nhttps://example.org\nhttps://example.net\n")
+            .write_stdin("../../issues\n")
             .env_clear()
             .assert()
             .success()
-            .stdout(contains("https://github.com/usnistgov/OSCAL/issues"))
-            .stdout(contains("http://127.0.0.1:8080/"))
-            .stdout(contains("https://staging.example.com/"))
-            .stdout(contains("https://example.net/"));
+            .stdout(contains("https://github.com/usnistgov/OSCAL/issues"));
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_remap_capture() -> Result<()> {
+        let mut cmd = main_command();
+
+        cmd.arg("--dump")
+            .arg("--remap")
+            .arg("https://example.com/(.*) http://example.org/$1")
+            .arg("--")
+            .arg("-")
+            .write_stdin("https://example.com/foo\n")
+            .env_clear()
+            .assert()
+            .success()
+            .stdout(contains("http://example.org/foo"));
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_remap_named_capture() -> Result<()> {
+        let mut cmd = main_command();
+
+        cmd.arg("--dump")
+            .arg("--remap")
+            .arg("https://github.com/(?P<org>.*)/(?P<repo>.*) https://gitlab.com/$org/$repo")
+            .arg("--")
+            .arg("-")
+            .write_stdin("https://github.com/lycheeverse/lychee\n")
+            .env_clear()
+            .assert()
+            .success()
+            .stdout(contains("https://gitlab.com/lycheeverse/lychee"));
 
         Ok(())
     }
@@ -1038,6 +1268,7 @@ mod cli {
 
         cmd.arg("--exclude-path")
             .arg(&excluded_path1)
+            .arg("--exclude-path")
             .arg(&excluded_path2)
             .arg("--")
             .arg(&test_path)
@@ -1089,4 +1320,242 @@ mod cli {
 
         Ok(())
     }
+
+    #[test]
+    fn test_prevent_too_many_redirects() -> Result<()> {
+        let mut cmd = main_command();
+        let url = "https://httpstat.us/308";
+
+        cmd.write_stdin(url)
+            .arg("--max-redirects")
+            .arg("0")
+            .arg("-")
+            .assert()
+            .failure();
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_suggests_url_alternatives() -> Result<()> {
+        for _ in 0..3 {
+            // This can be flaky. Try up to 3 times
+            let mut cmd = main_command();
+            let input = fixtures_path().join("INTERNET_ARCHIVE.md");
+
+            cmd.arg("--no-progress").arg("--suggest").arg(input);
+
+            // Run he command and check if the output contains the expected
+            // suggestions
+            let assert = cmd.assert();
+            let output = assert.get_output();
+
+            // We're looking for a suggestion that
+            // - starts with http://web.archive.org/web/
+            // - ends with google.com/jobs.html
+            let re = Regex::new(r"http://web\.archive\.org/web/.*google\.com/jobs\.html").unwrap();
+            if re.is_match(&String::from_utf8_lossy(&output.stdout)) {
+                // Test passed
+                return Ok(());
+            } else {
+                // Wait for a second before retrying
+                std::thread::sleep(std::time::Duration::from_secs(1));
+            }
+        }
+
+        // If we reached here, it means the test did not pass after multiple attempts
+        Err("Did not get the expected command output after multiple attempts.".into())
+    }
+
+    #[tokio::test]
+    async fn test_basic_auth() -> Result<()> {
+        let username = "username";
+        let password = "password123";
+
+        let mock_server = wiremock::MockServer::start().await;
+        Mock::given(basic_auth(username, password))
+            .respond_with(ResponseTemplate::new(200))
+            .mount(&mock_server)
+            .await;
+
+        // Configure the command to use the BasicAuthExtractor
+        main_command()
+            .arg("--verbose")
+            .arg("--basic-auth")
+            .arg(format!("{} {username}:{password}", mock_server.uri()))
+            .arg("-")
+            .write_stdin(mock_server.uri())
+            .assert()
+            .success()
+            .stdout(contains("1 Total"))
+            .stdout(contains("1 OK"));
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_multi_basic_auth() -> Result<()> {
+        let username1 = "username";
+        let password1 = "password123";
+        let mock_server1 = wiremock::MockServer::start().await;
+        Mock::given(basic_auth(username1, password1))
+            .respond_with(ResponseTemplate::new(200))
+            .mount(&mock_server1)
+            .await;
+
+        let username2 = "admin_user";
+        let password2 = "admin_pw";
+        let mock_server2 = wiremock::MockServer::start().await;
+
+        Mock::given(basic_auth(username2, password2))
+            .respond_with(ResponseTemplate::new(200))
+            .mount(&mock_server2)
+            .await;
+
+        // Configure the command to use the BasicAuthExtractor
+        main_command()
+            .arg("--verbose")
+            .arg("--basic-auth")
+            .arg(format!("{} {username1}:{password1}", mock_server1.uri()))
+            .arg("--basic-auth")
+            .arg(format!("{} {username2}:{password2}", mock_server2.uri()))
+            .arg("-")
+            .write_stdin(format!("{}\n{}", mock_server1.uri(), mock_server2.uri()))
+            .assert()
+            .success()
+            .stdout(contains("2 Total"))
+            .stdout(contains("2 OK"));
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_cookie_jar() -> Result<()> {
+        // Create a random cookie jar file
+        let cookie_jar = NamedTempFile::new()?;
+
+        let mut cmd = main_command();
+        cmd.arg("--cookie-jar")
+            .arg(cookie_jar.path().to_str().unwrap())
+            .arg("-")
+            // Using Google as a test target because I couldn't
+            // get the mock server to work with the cookie jar
+            .write_stdin("https://google.com/ncr")
+            .assert()
+            .success();
+
+        // check that the cookie jar file contains the expected cookies
+        let file = std::fs::File::open(cookie_jar.path()).map(std::io::BufReader::new)?;
+        let cookie_store = reqwest_cookie_store::CookieStore::load_json(file).unwrap();
+        let all_cookies = cookie_store.iter_any().collect::<Vec<_>>();
+
+        assert!(!all_cookies.is_empty());
+        assert!(all_cookies.iter().all(|c| c.domain() == Some("google.com")));
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_dump_inputs_glob_md() -> Result<()> {
+        let pattern = fixtures_path().join("**/*.md");
+
+        let mut cmd = main_command();
+        cmd.arg("--dump-inputs")
+            .arg(pattern)
+            .assert()
+            .success()
+            .stdout(contains("fixtures/dump_inputs/subfolder/file2.md"))
+            .stdout(contains("fixtures/dump_inputs/markdown.md"));
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_dump_inputs_glob_all() -> Result<()> {
+        let pattern = fixtures_path().join("**/*");
+
+        let mut cmd = main_command();
+        cmd.arg("--dump-inputs")
+            .arg(pattern)
+            .assert()
+            .success()
+            .stdout(contains("fixtures/dump_inputs/subfolder/test.html"))
+            .stdout(contains("fixtures/dump_inputs/subfolder/file2.md"))
+            .stdout(contains("fixtures/dump_inputs/subfolder"))
+            .stdout(contains("fixtures/dump_inputs/markdown.md"))
+            .stdout(contains("fixtures/dump_inputs/subfolder/example.bin"))
+            .stdout(contains("fixtures/dump_inputs/some_file.txt"));
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_dump_inputs_url() -> Result<()> {
+        let mut cmd = main_command();
+        cmd.arg("--dump-inputs")
+            .arg("https://example.com")
+            .assert()
+            .success()
+            .stdout(contains("https://example.com"));
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_dump_inputs_path() -> Result<()> {
+        let mut cmd = main_command();
+        cmd.arg("--dump-inputs")
+            .arg("fixtures")
+            .assert()
+            .success()
+            .stdout(contains("fixtures"));
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_dump_inputs_stdin() -> Result<()> {
+        let mut cmd = main_command();
+        cmd.arg("--dump-inputs")
+            .arg("-")
+            .assert()
+            .success()
+            .stdout(contains("Stdin"));
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_fragments() {
+        let mut cmd = main_command();
+        let input = fixtures_path().join("fragments");
+
+        cmd.arg("--verbose")
+            .arg("--include-fragments")
+            .arg(input)
+            .assert()
+            .failure()
+            .stderr(contains("fixtures/fragments/file1.md#fragment-1"))
+            .stderr(contains("fixtures/fragments/file1.md#fragment-2"))
+            .stderr(contains("fixtures/fragments/file1.md#code-heading"))
+            .stderr(contains("fixtures/fragments/file2.md#custom-id"))
+            .stderr(contains("fixtures/fragments/file1.md#missing-fragment"))
+            .stderr(contains("fixtures/fragments/file2.md#fragment-1"))
+            .stderr(contains("fixtures/fragments/file1.md#kebab-case-fragment"))
+            .stderr(contains(
+                "fixtures/fragments/file1.md#lets-wear-a-hat-%C3%AAtre",
+            ))
+            .stderr(contains("fixtures/fragments/file2.md#missing-fragment"))
+            .stderr(contains("fixtures/fragments/empty_file#fragment"))
+            .stderr(contains("fixtures/fragments/file.html#a-word"))
+            .stderr(contains("fixtures/fragments/file.html#in-the-beginning"))
+            .stderr(contains("fixtures/fragments/file.html#in-the-end"))
+            .stderr(contains(
+                "fixtures/fragments/file1.md#kebab-case-fragment-1",
+            ))
+            .stdout(contains("15 Total"))
+            .stdout(contains("12 OK"))
+            // 3 failures because of missing fragments
+            .stdout(contains("3 Errors"));
+    }
 }
