@@ -1,9 +1,11 @@
 use crate::{
-    extract::Extractor, helpers::request, types::uri::raw::RawUri, Base, Input, Request, Result,
+    basic_auth::BasicAuthExtractor, extract::Extractor, types::uri::raw::RawUri, utils::request,
+    Base, Input, Request, Result,
 };
+use futures::TryStreamExt;
 use futures::{
     stream::{self, Stream},
-    StreamExt, TryStreamExt,
+    StreamExt,
 };
 use par_stream::ParStreamExt;
 
@@ -11,10 +13,11 @@ use par_stream::ParStreamExt;
 /// It drives the link extraction from inputs
 #[derive(Debug, Clone)]
 pub struct Collector {
-    base: Option<Base>,
+    basic_auth_extractor: Option<BasicAuthExtractor>,
     skip_missing_inputs: bool,
     include_verbatim: bool,
     use_html5ever: bool,
+    base: Option<Base>,
 }
 
 impl Collector {
@@ -22,10 +25,11 @@ impl Collector {
     #[must_use]
     pub const fn new(base: Option<Base>) -> Self {
         Collector {
-            base,
+            basic_auth_extractor: None,
             skip_missing_inputs: false,
-            use_html5ever: false,
             include_verbatim: false,
+            use_html5ever: false,
+            base,
         }
     }
 
@@ -50,6 +54,24 @@ impl Collector {
         self
     }
 
+    /// Pass a [`BasicAuthExtractor`] which is capable to match found
+    /// URIs to basic auth credentials. These credentials get passed to the
+    /// request in question.
+    #[must_use]
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn basic_auth_extractor(mut self, extractor: BasicAuthExtractor) -> Self {
+        self.basic_auth_extractor = Some(extractor);
+        self
+    }
+
+    /// Collect all sources from a list of [`Input`]s. For further details,
+    /// see also [`Input::get_sources`](crate::Input#method.get_sources).
+    pub fn collect_sources(self, inputs: Vec<Input>) -> impl Stream<Item = Result<String>> {
+        stream::iter(inputs)
+            .par_then_unordered(None, move |input| async move { input.get_sources() })
+            .flatten()
+    }
+
     /// Fetch all unique links from inputs
     /// All relative URLs get prefixed with `base` (if given).
     /// (This can be a directory or a base URL)
@@ -57,24 +79,25 @@ impl Collector {
     /// # Errors
     ///
     /// Will return `Err` if links cannot be extracted from an input
-    pub async fn collect_links(self, inputs: Vec<Input>) -> impl Stream<Item = Result<Request>> {
+    pub fn collect_links(self, inputs: Vec<Input>) -> impl Stream<Item = Result<Request>> {
         let skip_missing_inputs = self.skip_missing_inputs;
-        let contents = stream::iter(inputs)
-            .par_then_unordered(None, move |input| async move {
-                input.get_contents(skip_missing_inputs).await
-            })
-            .flatten();
-
         let base = self.base;
-        contents
+        stream::iter(inputs)
+            .par_then_unordered(None, move |input| async move {
+                input.get_contents(skip_missing_inputs)
+            })
+            .flatten()
             .par_then_unordered(None, move |content| {
                 // send to parallel worker
                 let base = base.clone();
+                let basic_auth_extractor = self.basic_auth_extractor.clone();
                 async move {
                     let content = content?;
+
                     let extractor = Extractor::new(self.use_html5ever, self.include_verbatim);
                     let uris: Vec<RawUri> = extractor.extract(&content);
-                    let requests = request::create(uris, &content, &base)?;
+
+                    let requests = request::create(uris, &content, &base, &basic_auth_extractor)?;
                     Result::Ok(stream::iter(requests.into_iter().map(Ok)))
                 }
             })
@@ -99,7 +122,7 @@ mod tests {
 
     // Helper function to run the collector on the given inputs
     async fn collect(inputs: Vec<Input>, base: Option<Base>) -> HashSet<Uri> {
-        let responses = Collector::new(base).collect_links(inputs).await;
+        let responses = Collector::new(base).collect_links(inputs);
         responses.map(|r| r.unwrap().uri).collect().await
     }
 
@@ -116,7 +139,7 @@ mod tests {
         let file_path = temp_dir.path().join("README");
         let _file = File::create(&file_path).unwrap();
         let input = Input::new(&file_path.as_path().display().to_string(), None, true, None)?;
-        let contents: Vec<_> = input.get_contents(true).await.collect::<Vec<_>>().await;
+        let contents: Vec<_> = input.get_contents(true).collect::<Vec<_>>().await;
 
         assert_eq!(contents.len(), 1);
         assert_eq!(contents[0].as_ref().unwrap().file_type, FileType::Plaintext);
@@ -126,7 +149,7 @@ mod tests {
     #[tokio::test]
     async fn test_url_without_extension_is_html() -> Result<()> {
         let input = Input::new("https://example.com/", None, true, None)?;
-        let contents: Vec<_> = input.get_contents(true).await.collect::<Vec<_>>().await;
+        let contents: Vec<_> = input.get_contents(true).collect::<Vec<_>>().await;
 
         assert_eq!(contents.len(), 1);
         assert_eq!(contents[0].as_ref().unwrap().file_type, FileType::Html);
@@ -277,10 +300,10 @@ mod tests {
 
         let input = Input {
             source: InputSource::String(
-                r#"This is [an internal url](@/internal.md)
+                "This is [an internal url](@/internal.md)
         This is [an internal url](@/internal.markdown)
         This is [an internal url](@/internal.markdown#example)
-        This is [an internal url](@/internal.md#example)"#
+        This is [an internal url](@/internal.md#example)"
                     .to_string(),
             ),
             file_type_hint: Some(FileType::Markdown),
@@ -356,7 +379,7 @@ mod tests {
     async fn test_email_with_query_params() {
         let input = Input {
             source: InputSource::String(
-                r#"This is a mailto:user@example.com?subject=Hello link"#.to_string(),
+                "This is a mailto:user@example.com?subject=Hello link".to_string(),
             ),
             file_type_hint: None,
             excluded_paths: None,
