@@ -1,36 +1,12 @@
 use crate::types::{uri::raw::RawUri, FileType, InputContent};
 
-mod html5ever;
-mod html5gum;
-mod markdown;
+pub mod html;
+pub mod markdown;
 mod plaintext;
 
 use markdown::extract_markdown;
 use plaintext::extract_plaintext;
 
-/// Check if the given element is in the list of preformatted ("verbatim") tags.
-///
-/// These will be excluded from link checking by default.
-// Including the <script> tag is debatable, but the alternative is to
-// have a separate list of tags which need a separate config setting and that
-// seems worse.
-pub(crate) fn is_verbatim_elem(name: &str) -> bool {
-    matches!(
-        name,
-        "code"
-            | "kbd"
-            | "listing"
-            | "noscript"
-            | "plaintext"
-            | "pre"
-            | "samp"
-            | "script"
-            | "textarea"
-            | "var"
-            | "xmp"
-    )
-}
-
 /// A handler for extracting links from various input formats like Markdown and
 /// HTML. Allocations should be avoided if possible as this is a
 /// performance-critical section of the library.
@@ -69,9 +45,9 @@ impl Extractor {
             FileType::Markdown => extract_markdown(&input_content.content, self.include_verbatim),
             FileType::Html => {
                 if self.use_html5ever {
-                    html5ever::extract_html(&input_content.content, self.include_verbatim)
+                    html::html5ever::extract_html(&input_content.content, self.include_verbatim)
                 } else {
-                    html5gum::extract_html(&input_content.content, self.include_verbatim)
+                    html::html5gum::extract_html(&input_content.content, self.include_verbatim)
                 }
             }
             FileType::Plaintext => extract_plaintext(&input_content.content),
@@ -86,9 +62,9 @@ mod tests {
 
     use super::*;
     use crate::{
-        helpers::url::find_links,
         test_utils::{load_fixture, mail, website},
         types::{FileType, InputContent, InputSource},
+        utils::url::find_links,
         Uri,
     };
 
@@ -113,21 +89,9 @@ mod tests {
         uris_html5gum
     }
 
-    #[test]
-    fn test_verbatim_matching() {
-        assert!(is_verbatim_elem("pre"));
-        assert!(is_verbatim_elem("code"));
-        assert!(is_verbatim_elem("listing"));
-        assert!(is_verbatim_elem("script"));
-    }
-
     #[test]
     fn verbatim_elem() {
-        let input = r#"
-        <pre>
-        https://example.com
-        </pre>
-        "#;
+        let input = "<pre>https://example.com</pre>";
         let uris = extract_uris(input, FileType::Html);
         assert!(uris.is_empty());
     }
@@ -194,7 +158,7 @@ mod tests {
 
     #[test]
     fn test_md_escape() {
-        let input = r#"http://msdn.microsoft.com/library/ie/ms535874\(v=vs.85\).aspx"#;
+        let input = r"http://msdn.microsoft.com/library/ie/ms535874\(v=vs.85\).aspx";
         let links: Vec<_> = find_links(input).collect();
         let expected = "http://msdn.microsoft.com/library/ie/ms535874(v=vs.85).aspx)";
 
