@@ -50,12 +50,14 @@
 #[cfg(doctest)]
 doc_comment::doctest!("../../README.md");
 
+mod basic_auth;
 mod client;
 /// A pool of clients, to handle concurrent checks
 pub mod collector;
-mod helpers;
 mod quirks;
+mod retry;
 mod types;
+mod utils;
 
 /// Functionality to extract URIs from inputs
 pub mod extract;
@@ -73,11 +75,14 @@ pub mod test_utils;
 
 #[cfg(test)]
 use doc_comment as _; // required for doctest
-use openssl_sys as _; // required for vendored-openssl feature
 use ring as _; // required for apple silicon
 
+#[cfg(feature = "native-tls")]
+use openssl_sys as _; // required for vendored-openssl feature
+
 #[doc(inline)]
 pub use crate::{
+    basic_auth::BasicAuthExtractor,
     // Constants get exposed so that the CLI can use the same defaults as the library
     client::{
         check, Client, ClientBuilder, DEFAULT_MAX_REDIRECTS, DEFAULT_MAX_RETRIES,
@@ -86,7 +91,8 @@ pub use crate::{
     collector::Collector,
     filter::{Excludes, Filter, Includes},
     types::{
-        uri::valid::Uri, Base, CacheStatus, ErrorKind, FileType, Input, InputContent, InputSource,
-        Request, Response, ResponseBody, Result, Status,
+        uri::valid::Uri, AcceptRange, AcceptRangeError, AcceptSelector, Base, BasicAuthCredentials,
+        BasicAuthSelector, CacheStatus, CookieJar, ErrorKind, FileType, Input, InputContent,
+        InputSource, Request, Response, ResponseBody, Result, Status,
     },
 };
