@@ -1,7 +1,10 @@
 #![allow(unreachable_pub)]
 
+mod accept;
 mod base;
+mod basic_auth;
 mod cache;
+mod cookies;
 mod error;
 mod file;
 mod input;
@@ -11,8 +14,11 @@ mod response;
 mod status;
 pub(crate) mod uri;
 
+pub use accept::*;
 pub use base::Base;
+pub use basic_auth::{BasicAuthCredentials, BasicAuthSelector};
 pub use cache::CacheStatus;
+pub use cookies::CookieJar;
 pub use error::ErrorKind;
 pub use file::FileType;
 pub use input::{Input, InputContent, InputSource};
